package java.security;

import sun.security.jca.JCAUtil;
import java.nio.ByteBuffer;
import java.security.spec.AlgorithmParameterSpec;

public abstract class SignatureSpi
{
    protected SecureRandom appRandom;
    
    public SignatureSpi() {
        this.appRandom = null;
    }
    
    protected abstract void engineInitVerify(PublicKey p0) throws InvalidKeyException;
    
    void engineInitVerify(PublicKey publicKey, AlgorithmParameterSpec algorithmParameterSpec) throws InvalidKeyException, InvalidAlgorithmParameterException {
        if (algorithmParameterSpec != null) {
            try {
                this.engineSetParameter(algorithmParameterSpec);
            }
            catch (UnsupportedOperationException ex) {
                throw new InvalidAlgorithmParameterException(ex);
            }
        }
        this.engineInitVerify(publicKey);
    }
    
    protected abstract void engineInitSign(PrivateKey p0) throws InvalidKeyException;
    
    protected void engineInitSign(PrivateKey privateKey, SecureRandom appRandom) throws InvalidKeyException {
        this.appRandom = appRandom;
        this.engineInitSign(privateKey);
    }
    
    void engineInitSign(PrivateKey privateKey, AlgorithmParameterSpec algorithmParameterSpec, SecureRandom secureRandom) throws InvalidKeyException, InvalidAlgorithmParameterException {
        if (algorithmParameterSpec != null) {
            try {
                this.engineSetParameter(algorithmParameterSpec);
            }
            catch (UnsupportedOperationException ex) {
                throw new InvalidAlgorithmParameterException(ex);
            }
        }
        this.engineInitSign(privateKey, secureRandom);
    }
    
    protected abstract void engineUpdate(byte p0) throws SignatureException;
    
    protected abstract void engineUpdate(byte[] p0, int p1, int p2) throws SignatureException;
    
    protected void engineUpdate(ByteBuffer byteBuffer) {
        if (!byteBuffer.hasRemaining()) {
            return;
        }
        try {
            if (byteBuffer.hasArray()) {
                byte[] array = byteBuffer.array();
                int arrayOffset = byteBuffer.arrayOffset();
                int position = byteBuffer.position();
                int limit = byteBuffer.limit();
                this.engineUpdate(array, arrayOffset + position, limit - position);
                byteBuffer.position(limit);
            }
            else {
                int i = byteBuffer.remaining();
                byte[] dst = new byte[JCAUtil.getTempArraySize(i)];
                while (i > 0) {
                    int min = Math.min(i, dst.length);
                    byteBuffer.get(dst, 0, min);
                    this.engineUpdate(dst, 0, min);
                    i -= min;
                }
            }
        }
        catch (SignatureException ex) {
            throw new ProviderException("update() failed", ex);
        }
        return;
    }
    
    protected abstract byte[] engineSign() throws SignatureException;
    
    protected int engineSign(byte[] array, int n, int n2) throws SignatureException {
        byte[] engineSign = this.engineSign();
        if (n2 < engineSign.length) {
            throw new SignatureException("partial signatures not returned");
        }
        if (array.length - n < engineSign.length) {
            throw new SignatureException("insufficient space in the output buffer to store the signature");
        }
        System.arraycopy(engineSign, 0, array, n, engineSign.length);
        return engineSign.length;
    }
    
    protected abstract boolean engineVerify(byte[] p0) throws SignatureException;
    
    protected boolean engineVerify(byte[] array, int n, int n2) throws SignatureException {
        byte[] array2 = new byte[n2];
        System.arraycopy(array, n, array2, 0, n2);
        return this.engineVerify(array2);
    }
    
    @Deprecated
    protected abstract void engineSetParameter(String p0, Object p1) throws InvalidParameterException;
    
    protected void engineSetParameter(AlgorithmParameterSpec algorithmParameterSpec) throws InvalidAlgorithmParameterException {
        throw new UnsupportedOperationException();
    }
    
    protected AlgorithmParameters engineGetParameters() {
        throw new UnsupportedOperationException();
    }
    
    @Deprecated
    protected abstract Object engineGetParameter(String p0) throws InvalidParameterException;
    
    public Object clone() throws CloneNotSupportedException {
        if ((this instanceof Cloneable)) {
            return super.clone();
        }
        throw new CloneNotSupportedException();
    }
}
